setup

Function setup 

pub async fn setup(pool: &PgPool) -> Result<(), BoxDynError>
Sets up the queue schema in the database.

It’s recommended to run this once during application startup.

Note: Separation from the main database is recommended but not required.