pub async fn setup_from_url(url: &str) -> Result<(), BoxDynError>Expand description
Sets up the queue schema in the database using a PostgreSQL URL.
pub async fn setup_from_url(url: &str) -> Result<(), BoxDynError>Sets up the queue schema in the database using a PostgreSQL URL.