start_without_janitor

Function start_without_janitor 

pub async fn start_without_janitor(jq: Arc<SimpleQueue>) -> JoinSet<()>
Starts the queue without a janitor task.

For cases when you don’t need a janitor task, i.e. completed and error queues should stay in the same table. Useful for development and testing.

It returns a tokio::task::JoinSet that can be used to wait for the tasks to complete, and it does so only AFTER the queue has started polling.