pub enum JobResult {
Success,
Failed,
RetryAt(DateTime<Utc>),
RescheduleAt(DateTime<Utc>),
HandlerMissing,
Critical,
Cancel,
Unprocessable,
InternalError,
}Expand description
Result of a job execution, should be returned by the job handler.
Variants
Success
Job processed successfully.
Failed
Job failed, subject to retry (with BackoffStrategy).
RetryAt(DateTime<Utc>)
Retry (i.e. attempt count incremented) at a specific time. IMPORTANT: Job cannot be retried sooner than the backoff strategy allows
RescheduleAt(DateTime<Utc>)
Reschedule to specific time. Attempt it NOT consumed. Use with caution. Queue’s max_reprocess_count is a safety measure to prevent infinite rescheduling.
HandlerMissing
Handler not found for the job type, status changes to unprocessable.
Critical
Critical failure, job is not retried.
Cancel
Cancel the job (i.e. do not retry anymore), status changes to cancelled.
Unprocessable
Handler can’t process the job (e.g. deserialization failures), status changes to unprocessable.
InternalError
Internal error, same as Failed but looks nicer when handler encountered recoverable error. Attempt is consumed.