pub struct Reaper {
pub heartbeat_interval: Interval,
pub pool: PgPool,
}Expand description
Reaper is a background task that fixes stale jobs.
It periodically runs and:
- Check for stale jobs (not updated for some time but status =
running) - Marks jobs as failed when their attempt count exceeds the maximum allowed
Fields
heartbeat_interval: Intervalpool: PgPoolImplementations