Module sync

Module sync 

Structs

InstantStrategy
Permit
Type erased permit that should be returned by JobStrategy

Enums

BackoffStrategy
JobStrategyError
When aquiring a permit from a JobStrategy, permit giver can return an error to impact on the job’s processing

Traits

JobStrategy
Trait for job strategies that define how to acquire final, 3rd permit