pub enum JobStrategyError {
CancelJob,
TryAfter(Duration),
Retry,
MarkCompleted,
}Expand description
When aquiring a permit from a JobStrategy, permit giver can return an error to impact on the job’s processing
Variants
CancelJob
The job shouldn’t be processed, mark as canceled (e.g. feature flags, resource no longer available, deprecated jobs etc.)
TryAfter(Duration)
The job should be tried again after a delay (e.g. resource temporarily unavailable, renewable limits, etc. )
Retry
Retry with backoff strategy (consumes attempt)
MarkCompleted
Mark job as completed without processing, e.g. when seasonal job backed off far into future is no longer relevant, but shouldn’t be sent to dead-letter queue