pub async fn start(jq: Arc<SimpleQueue>) -> JoinSet<()>Expand description
Queue starting function.
It starts:
- A reaper task that reclaims stalled jobs
- A janitor task that periodically checks the queue, archives old jobs and moves errored jobs to a dead queue
It returns a tokio::task::JoinSet that can be used to wait for the tasks to complete,
and it does so only AFTER the queue has started polling.